<?php

/**
 * All of our widgets load via ajax. This is for two reasons:
 * - It diminishes the effect of a long page load if there is no cache yet.
 * - It is a step on the path to listening to JS events from the Ecwid storefront.
 *
 * @package WordPress
 * @subpackage sjf-et
 * @since SJF ET 1.5.5
 */

function sjf_et_ajax_init() {
	new SJF_Ecwid_Ajax();
}
add_action( 'plugins_loaded', 'sjf_et_ajax_init' );

/**
 * Provides static methods that widgets will use to format their output,
 * and non-static methods for generating widget HTML at a url that can be jQuery loaded.
 */
class SJF_Ecwid_Ajax {

	/**
	 * Add actions for our class.
	 */
	public function __construct() {

		// We need to wait until init before we add a feed.
		add_action( 'init', array( $this, 'init' ) );

	}

	/**
	 * Register our feed with WordPress.
	 */
	public function init() {
		add_feed( self::get_url(), array( $this, 'the_html' ) );
	}

	/**
	 * Build a small JS snippet for each widget instance to ajax load it.
	 * 
	 * @param  string $which_widget Which type of widget are we calling?
	 * @param  string $widget_id    Which specific instance of the widget is it?
	 * 
	 * @return string A small JS snippet for each widget instance to ajax load it.
	 */
	public static function get_script( $which_widget, $widget_id, $args = array() ) {
		
		// Build the url that will publish the HTML for this widget.
		$url = get_bloginfo( 'url' ) . '?feed=' . self::get_url();
		$url = add_query_arg( array( 'which_widget' => $which_widget ), $url );

		if( isset( $args['which_product'] ) ) {
			$which_product = sanitize_text_field( $args['which_product'] );
			$url           = add_query_arg( array( 'which_product' => $which_product ), $url );
		}

		if( isset( $args['which_products'] ) ) {
			$which_products = sanitize_text_field( $args['which_products'] );
			$url            = add_query_arg( array( 'which_products' => $which_products ), $url );
		}

		if( isset( $args['which_categories'] ) ) {
			$which_categories = sanitize_text_field( $args['which_categories'] );
			$url              = add_query_arg( array( 'which_categories' => $which_categories ), $url );
		}

		if( isset( $args['label'] ) ) {
			$label = $args['label'];
			$url   = add_query_arg( array( 'label' => $label ), $url );
		}

		if( isset( $args['image_size'] ) ) {
			$image_size = sanitize_text_field( $args['image_size'] );
			$url        = add_query_arg( array( 'image_size' => $image_size ), $url );
		}

		if( isset( $args['widget_title'] ) ) {
			$widget_title = $args['widget_title'];
			$url          = add_query_arg( array( 'widget_title' => $widget_title ), $url );
		}

		if( isset( $args['order'] ) ) {
			$order = $args['order'];
			$url   = add_query_arg( array( 'order' => $order ), $url );
		}

		if( isset( $args['order_by'] ) ) {
			$order_by = $args['order_by'];
			$url      = add_query_arg( array( 'order_by' => $order_by ), $url );
		}

		if( isset( $args['enabled'] ) ) {
			$enabled = $args['enabled'];
			$url     = add_query_arg( array( 'enabled' => $enabled ), $url );
		}

		if( isset( $args['hidden_categories'] ) ) {
			$hidden_categories = $args['hidden_categories'];
			$url               = add_query_arg( array( 'hidden_categories' => $hidden_categories ), $url );
		}

		if( isset( $args['limit'] ) ) {
			$limit = $args['limit'];
			$url   = add_query_arg( array( 'limit' => $limit ), $url );
		}						

		$url = esc_url_raw( $url );

		$out = <<<EOT
			<script>

				/**
				 * Ajax this widget into the page, inside the loader div.
				 */
				jQuery( document ).ready( function( $ ) {

					// The url that hosts the HTML for this widget.
					var url = '$url';

					// The unique ID for this widget instance.
					var widgetID = '$widget_id';
					
					// Might hold the data for the current Ecwid store page.
					var page = false;

					// If we are on the store page, and are trying to reflect that in our widgets...
					if( SJF_ET.isStore() && SJF_ET.requestingCurrent( url ) ) {

						// Every time the store loads...
						Ecwid.OnPageLoad.add( function( page ) {
					
							SJF_ET.load( url, page, widgetID );
				
						});
					
					// Else, we can just load the widget, no need to check on the state of the store.
					} else {
					
						SJF_ET.load( url, page, widgetID );

					}

				});

			</script>
EOT;

		return $out;

	}

	/**
	 * Get the url that publishes html for any of our widgets.
	 * 
	 * @return string The url that publishes html for any of our widgets
	 */
	public static function get_url() {
		return 'widgets.html';
	}

	/**
	 * Get the wrapper div into which our widgets load.
	 * 
	 * @param  string $widget_id The ID for a specific widget instance.
	 * @return string The wrapper div into which our widgets load.
	 */
	public static function get_load_el( $widget_id ) {
		return "<div class='$widget_id-load'></div>";
	}

	/**
	 * The main template tag for drawing our feed.
	 */
	public function the_html() {
		
		$args = array();

		if( ! isset( $_GET['which_widget'] ) ) { return FALSE; }
		$which_widget = $_GET['which_widget'];

		if( ! class_exists( $which_widget ) ) { return FALSE; }

		if( ! method_exists( $which_widget, 'get_html' ) ) { return FALSE; }

		$widget = new $which_widget;

		if( isset( $_GET['widget_title'] ) ) {
			$widget_title        = wp_kses_post( stripslashes( urldecode( $_GET['widget_title'] ) ) );
			$args['widget_title']= $widget_title;
		}

		if( isset( $_GET['which_product'] ) ) {
			$which_product        = sanitize_text_field( $_GET['which_product'] );
			$args['which_product']= $which_product;
		}

		if( isset( $_GET['which_products'] ) ) {
			$which_products        = sanitize_text_field( $_GET['which_products'] );
			$args['which_products']= $which_products;
		}

		if( isset( $_GET['which_categories'] ) ) {
			$which_categories        = sanitize_text_field( $_GET['which_categories'] );
			$args['which_categories']= $which_categories;
		}

		if( isset( $_GET['image_size'] ) ) {
			$image_size        = sanitize_text_field( $_GET['image_size'] );
			$args['image_size']= $image_size;
		}

		if( isset( $_GET['label'] ) ) {
			$label        = wp_kses_post( stripslashes( urldecode( $_GET['label'] ) ) );
			$args['label']= $label;
		}

		if( isset( $_GET['order'] ) ) {
			$order        = sanitize_text_field( $_GET['order'] );
			$args['order']= $order;
		}	

		if( isset( $_GET['order_by'] ) ) {
			$order_by        = sanitize_text_field( $_GET['order_by'] );
			$args['order_by']= $order_by;
		}	

		if( isset( $_GET['enabled'] ) ) {
			$enabled        = sanitize_text_field( $_GET['enabled'] );
			$args['enabled']= $enabled;
		}		

		if( isset( $_GET['hidden_categories'] ) ) {
			$hidden_categories        = sanitize_text_field( $_GET['hidden_categories'] );
			$args['hidden_categories']= $hidden_categories;
		}

		if( isset( $_GET['limit'] ) ) {

			$limit        = sanitize_text_field( $_GET['limit'] );
			$args['limit']= $limit;
		}	

		// It's assumed that all of our widgets will output their html with a funciton named get_html.
		echo $widget -> get_html( $args );

	} 

}