/**
 * Scripts for our plugin widgets.
 * 
 * @package WordPress
 * @subpackage sjf-et
 * @since SJF ET 1.3
 */

/**
 * Our class for dealing with the Ecwid storefront.
 * 
 * @type {object}
 */
var SJF_ET = SJF_ET || {};

SJF_ET = {

	/**
	 * Determine if the current page contains the Ecwid storefront.
	 * 
	 * @return {Boolean} If the current page contains an Ecwid store, return true.
	 */
	isStore: function() {

		// Does Ecwid exist?
		if( typeof Ecwid != 'object' ) { return false; }
		
		// Does the Ecwid JS API exist in the way we are expecting?
		if( jQuery.type( Ecwid.OnPageLoad ) != 'object' ) { return false; }		
	
		return true;
	
	},

	/**
	 * Are we on an Ecwid page of some kind?
	 * 
	 * @param  {Object}  page An Ecwid strorefront page.
	 * @return {Boolean} If the current page is an ecwid page, return true, else false.
	 */
	isPage: function ( page ) {
		
		// First of all, are we on the storefront?
		if( ! this.isStore() ) { return false; }

		// Are we on a page?
		if( jQuery.type( page ) != 'object' ) { return false; }

		// Is the page weird?
		if( jQuery.type( page.type ) != 'string' ) { return false; }

		return true;

	},

	/**
	 * Are we currently browsing an Ecwid product category?
	 * 
	 * @param  {Object}  page The Ecwid page.
	 * @return {Boolean} If the current page is an ecwid category page, return true, else false.
	 */
	isCategory: function( page ) {

		// If we're not on an ecwid page, bail.
		if( ! this.isPage( page ) ) { return false; }

		// If we're not on a category, bail.
		if( page.type != 'CATEGORY' ) { return false; }

		// If the category ID is weird, bail.
		if( typeof page.categoryId === 'undefined' ) { return false; }	

		// If it's 0, bail -- this would indicate we are on the home page.
		if( page.categoryId === 0 ) { return false; }

		return true;

	},

	/**
	 * Are we on the Ecwid store front page?
	 * @param  {Object}  page The Ecwid page.
	 * @return {Boolean} If the current page is an ecwid storefront page, return true, else false.
	 */
	isIndex: function( page ) {
			
		// Are we on a page?
		if( ! this.isPage( page ) ) { return false; }

		// Are we on a category?
		if( this.isCategory( page ) ) { return false; }
		
		// Are we on a product?
		if( this.isProduct( page ) ) { return false; }

		// Is there a category ID?  There should be.
		if( typeof page.categoryId === 'undefined' ) { return false; }
		
		// It should equal 0.
		if( page.categoryId != 0 ) { return false; }

		return true;

	},

	/**
	 * Are we on an Ecwid product page?
	 * 
	 * @param  {Object}  page The Ecwid page.
	 * @return {Boolean} If the current page is an ecwid product page, return true, else false.
	 */
	isProduct: function( page ) {
	
		// Are we on a page?
		if( ! this.isPage( page ) ) { return false; }

		// Is it a product page?
		if( page.type != 'PRODUCT' ) { return false; }

		return true;

	},

	/**
	 * Get the category ID for the current view.
	 * 
	 * @param  {Object} page The Ecwid page.
	 * @return {Number} If the current page is an ecwid product or category page, return the cat ID, else false.
	 */
	getCategory: function( page ) {
		
		// If we are on a category, grab the category ID.
		if( this.isCategory( page ) ) {
			return page.categoryId;
		}

		// If we are on a product, grab the category ID.
		if( this.isProduct( page ) ) {
			return page.categoryId;
		}

		return false;

	},

	/**
	 * Get the ID of the category currently being viewed.
	 * 
	 * @param  {Object} page The Ecwid page.
	 * @return {Number} If the current page is an ecwid product page, return the product ID, else false.
	 */
	getProduct: function( page ) {
		
		// Are we on a product?
		if( ! this.isProduct( page ) ) { return false; }

		return page.productId;

	},

	/**
	 * Parse a url to react to the user navigate around the Ecwid store.
	 * 
	 * @param  {String} url  The url for the widget HTML we are ajax-loading.
	 * @param  {Object} page The Ecwid page.
	 * @return {String} The url for the widget HTML we are ajax-loading.
	 */
	parseURL: function( url, page ) {

		// Is the widget configured to grab products from the current category?
		if( this.requestingCurrent( url ) ) {

			// If so, adjust the url to include a request for the current category.
			url = this.currentifyURL( url, page );
		}

		// Sanitize the category IDs.
		url = this.sanitizeCats( url );

		// Make srue the category ID's are unique.
		url = this.uniqueifyCats( url );
		
		return url;

	},

	/**
	 * Eliminate duplicate values from a url query.
	 * 
	 * @param  {String} url  The url who query params we are making unique.
	 * @return {String} The url with duplicate values removed.
	 */
	uniqueifyCats: function( url ) {

		// Grab the cats from the url.
		var cats = this.getCatIDs( url );
		
		// This will hold unique values from the input cats.
		var uniqueCats = [];

		// For each cat given to us...
		jQuery( cats ).each( function( index, value ) {

			// Is it in the output array yet?
			var inArray = jQuery.inArray( value, uniqueCats );
			
			// If not, add it.
			if( inArray == -1 ) {
				uniqueCats.push( value );
			}

		});

		// Conver the unique cats back to a string.
		uniqueCats = uniqueCats.toString();

		// Add the cats back ot the url.
		var out = this.addQueryArg( 'which_categories', uniqueCats, url );

		return out;

	},

	/**
	 * Eliminate weird values from a url query.
	 * 
	 * @param  {String} url  The url who query params we are making unique.
	 * @return {String} The url with duplicate values removed.
	 */
	sanitizeCats: function( url ) {

		// Grab the cats from the url.
		var cats = this.getCatIDs( url );
		
		// This will hold clean cats.
		var sanCats = [];

		// For each cat given to us...
		jQuery( cats ).each( function( index, value ) {

			// Is it the string, false?
			if( value == 'false' ) {

				// If so, skip it.
				return true;
			}

			// We made it!  add it to the output.
			sanCats.push( value );

		});

		// Convert the clean cats back to a string.
		sanCats = sanCats.toString();
		
		// Add the clean cats to the url.
		var out = this.addQueryArg( 'which_categories', sanCats, url );

		return out;

	},

	/**
	 * Adjust a url to include a request for the current category.
	 * 
	 * @param  {String} url  The url for the widget HTML that we are ajax-loading.
	 * @param  {Object} page The Ecwid page.
	 * @return {String} The url for the widget HTML, to include the ID for the current category.
	 */
	currentifyURL: function( url, page ) {

		// Grab the current category ID for the Ecwid store.
		var current = this.getCategory( page );

		/**
		 * Adjust the category ID's that are in the widget url, such that the keyword for 'current' is replaced by the cat ID for the current category.
		 */
		var catIDs    = this.replaceCatIDs( 'current', current, url );
		var catIDsStr = catIDs.toString();

		// Change the url such that the arg for which_categories includes the current category.
		var url = this.addQueryArg( 'which_categories', catIDsStr, url );

		return url;

	},

	/**
	 * Add a query arg to a url.  If the arg is already in the url, it will be updated.
	 * 
	 * @param {String} key   The key for the url param that we are adding.
	 * @param {String} value The value for the url param that we are adding.
	 * @param {String} url   The url to wich we are adding a query arg.
	 */
	addQueryArg: function( key, value, url ) {

		// Build the first part of the url -- the part without a query string.
		var base = this.getBaseURL( url );

		// Grab the query string as an object.
		var queryObj = this.getQueryObj( url );

		// Update the query to include the value being passed in to our function here.
		queryObj[key]=value;

		// Convert the query object back to a string.
		var queryStr = decodeURIComponent( jQuery.param( queryObj ) );
		
		// Build the url, by adding the query string to the base.
		var out = base + '?' + queryStr;

		return out;

	},

	/**
	 * Swap out a category ID in a url query arg with a different one.
	 * 
	 * @param  {String} remove The cat ID we are removing.
	 * @param  {String} add    The cat ID we are adding.
	 * @param  {String} url    The url we are manipulating.
	 * @return {Object}        An array of cat ID's, adjusted.
	 */
	replaceCatIDs: function( remove, add, url ) {

		// Grab the category ID's from the url.
		var ids = this.getCatIDs( url );

		// Start the output var by adding the one we need to add.  Make sure it's a string.
		if( jQuery.type( add ) == 'number' ) { add = add.toString(); }
		var out = [add];

		// Foreach id...
		jQuery( ids ).each( function( index, value ) {

			// If it's the one we want to remove, skip it.
			if( value === remove ) {
				return true;

			// Else, add it to the output var.
			} else {
				out.push( value );
			}

		});

		// Order the cats.
		out.sort();

		return out;

	},

	/**
	 * Break a url into it's component parts.
	 * 
	 * @param  {String} url A url.
	 * @return {Object} The url, broken into parts.
	 */
	getURLAsObject: function( url ) {

		// Start a new object.
		var out = {};

		// Create a jquery link object with our url so that we can dissect it.
		var a = jQuery( '<a>', { href:url } )[0];

		// Grab each part of the url into our output var.
		out.protocol = ( a.protocol );
		out.host     = ( a.host );
		out.hostname = ( a.hostname );
		out.port     = ( a.port );
		out.pathname = ( a.pathname );
		out.search   = ( a.search );
		out.hash     = ( a.hash );

		return out;

	},

	/**
	 * Get the query portion of a url, as an object.
	 * 
	 * @param  {String} url A url.
	 * @return {Object} The key/value pairs from the url query, as an object.
	 */
	getQueryObj: function( url ) {

		// Start our output var as an empty object.
		var out = {};

		// Convert the url to an object.
		var urlObj = this.getURLAsObject( url );
		
		// Grab the query string.
		var search = urlObj.search;

		// Remove the leading question mark from the query string.
		if ( search.substring( 0, 1 ) === '?') { 
			search = search.substring( 1 );
		}

		// Split the query string into key/value pairs
    	var queries = search.split( '&' );

		// Convert the array of strings into an object
    	jQuery( queries ).each( function( index, value ) {

    		// So, value is 'page=home' or similar.
    		var query = value.split( '=' );

    		// key would be 'page' in our example.
    		var key = query[0];

    		// val would be 'home' in our example.
    		var val = query[1];

    		// Assign the key/val to our output var.
    		out[key]=val;

    	});

    	return out;

	},

	getBaseURL: function( url ) {
		
		// Convert the url into an object.
		var obj = this.getURLAsObject( url );

		// Build the first part of the url -- the part without a query string.
		var base = obj.protocol + '//' + obj.hostname + obj.pathname;
	
		return base;

	},

	/**
	 * Get the cat ID's from a url.
	 * 
	 * @param  {String} url A url.
	 * @return {Object} The category ID's as an array.
	 */
	getCatIDs: function( url ) {
		
		// Grab the query string as an object.
		var obj = this.getQueryObj( url );
	
		// Grab the category ID's from the query.
		var which_categories = obj.which_categories;
		if( typeof which_categories === 'undefined' ) { return false; }

		// Convert the categories into an array.
		var catIDs = which_categories.split( ',' );

		var out = [];

		jQuery( catIDs ).each( function( index, value ) {

			// This just makes it easier though I don't remember why.
			value = value.toString();
				
			out.push( value );
			
		});

		out.sort();

		return out;

	},

	/**
	 * Determine to which widget a url is referring.
	 * 
	 * @param  {String} url A url.
	 * @return {String} A widget class name.
	 */
	getWhichWidget: function( url ) {

		// Get the query string as an object.
		var obj = this.getQueryObj( url );
	
		// Find the which_widget arg in the query.
		var out =  obj.which_widget;
		if( typeof out === 'undefined' ) { return false; }

		return out;

	},

	/**
	 * Determine the CSS class for a widget.
	 * 
	 * @param  {String} whichWidget The unique WP id for a widget instance.
	 * @param  {String} context     Do we want to output this for markup (HTML), or for selecting (CSS/JS)?
	 * @return {String} The css class for this widget.
	 */
	getChildClass: function( whichWidget, context ) {

		if( jQuery.type( whichWidget ) !== 'string' ) { return false; }

		// Strip non lowercase & underscores.
		var out = whichWidget.replace( /[^a-z_]/g, '' );
		
		// Strip trailing underscore that might come with shortcode versions of widgets.
		out = out.replace( /_+$/, '' );

		// If this is not for markup, prepend a period.
		if( ( context != 'markup' ) ) {
			out = '.' + out;
		}

		return out;

	},

	/**
	 * Is the widget looking to load content the refers to the current Ecwid page?
	 * 
	 * @param  {String}  url The url of a widget html output.
	 * @return {Boolean} If the widget is looking for 'current' content, return true, else false.
	 */
	requestingCurrent: function( url ) {

		var out = false;

		// Grab the category ID's from the widget url.
		var ids = this.getCatIDs( url );

		// For each cat id...
		jQuery( ids ).each( function( index, value ){

			// Is one of them sking for current?
			if( value === 'current' ) {
				
				out = true;

				// Brak out of the loop.
				return false;
			}
			
		});

		return out;

	},

	needNewRequest: function( oldUrl, newUrl ) {

		if( oldUrl === newUrl ) { return false; }

		return true;

	},

	/**
	 * Load the html for a widget into the container for a widget.
	 * 
	 * @param  {String} url      The url for a widget HTML.
	 * @param  {String} page     The current Ecwid page, used to grab content that corresponds to the Ecwid view.
	 * @param  {String} widgetID The WordPress ID for this widget instance.
	 */
	load: function( url, page, widgetID ) {

		// Parse the url such that it grabs content from the current page.
		var parsedUrl = this.parseURL( url, page );

		// For widgets that track the current Ecwid page, do we need to make a new request or do we already have the HTML we'd need?
		if( page && ! this.needNewRequest( this.currentUrl, parsedUrl ) ) {
			return false;
		}

		// Set our "class property" (if you will) which keeps track of the *previous* url.
		this.currentUrl = parsedUrl;

		// Which widget are we grabbing?
		var whichWidget = this.getWhichWidget( parsedUrl );
		
		// What, then, is the name of the callback function we are running after the widget is loaded?
		var CB = whichWidget + 'CB';

		// Load the widget.
		jQuery( '.' + widgetID + '-load' ).load( parsedUrl, function() {

			// Once it's loaded, run a callback function.
			var fn = window[CB];
			if( typeof fn != 'undefined' ) {
				
				// Pass the callback function an arg -- the widget ID.
				fn( '#' + widgetID );
			}

			// Fade it in.
			jQuery( '.' + widgetID + '-load' ).hide().fadeIn()

		});

	},

	/**
	 * Holds the url for the current widget HTML, so we can detect if we're requesting new content.
	 * @type {String}
	 */
	currentEcwidPage: ''

};

/**
 * Accordion
 */
function SJF_ET_AccordionCB( widgetID ) {
	
	/**
	 * We do defaults for this widget so it's easy to call for wp-admin modules or other non-widget uses.
	 */

	var childClass = SJF_ET.getChildClass( widgetID );

	// Default value for id is the body element.
	if( typeof widgetID === 'undefined') {
		widgetID = 'body'; 
	}

	// Default value for child class is '.sjf_et_accordion'.
	if( jQuery.type( childClass ) != 'string' ) {
		childClass = '.sjf_et_accordion';
	}

	// Hide all elements that carry our plugin hide class.
	var hide = jQuery( widgetID ).find( childClass );
	jQuery( hide ).hide();

	// Our plugin toggler class.  When it's clicked, it's hidden slblings are revealed.
	var toggle = jQuery( widgetID ).find( '.sjf_et-toggle' );
	jQuery( toggle ).click( function( event ) {
		event.preventDefault();
		jQuery( this ).siblings( childClass ).slideToggle();

		// We typically have a toggle arrow, which we'll invert on open/close.
		jQuery( this ).find( '.dashicons' ).toggleClass( 'dashicons-arrow-down-alt dashicons-arrow-up-alt' );
	});

}

/**
 * Autosuggest
 *
 * The args were localized in the Autosuggest widget file.
 */
function SJF_ET_AutosuggestCB( widgetID ) {
	
	if( typeof SJF_ET_Autosuggest === 'undefined' ) { return false; }
	if( ! jQuery().autocomplete ) { return false; }


	// The configuration for the autosuggest.
	var source         = SJF_ET_Autosuggest.source;
	var errorText      = SJF_ET_Autosuggest.error_text;
	var childClass     = SJF_ET.getChildClass( widgetID );
	var childClassHTML = SJF_ET.getChildClass( widgetID, 'markup' );

	// Use the source to build an array of valid labels for pointing the form to.
	var haystack = [];
	jQuery( source ).each( function( index, value ) {
		haystack.push( value.label );
	});

	// Instantiate the autocomplete.
	jQuery( widgetID ).find( childClass + '-input' ).autocomplete({

		// No min length.
		minLength: 0,

		// All the products.
		source: source,

		// Show the results in our widget.
		appendTo: widgetID + ' ' + childClass + '-suggestions',
		
		// Position them at the left bottom of the widget.
		position: { my : 'left top', at: 'left bottom' },

		// When an item is focused, don't do anything just yet.
		focus: function( event, ui ) {
			return false;
		},

		// When an item is selected, use the label for the input and the href for the form action.  And submit the form!
		select: function( event, ui ) {

			// Grab the label for this item and toss it in the search box.
			jQuery( widgetID ).find( childClass + '-input' ).val( ui.item.label );

			// Grab the href and point the form to it.
			jQuery( widgetID ).find( childClass ).attr({ action: ui.item.value });
			
			// Submit the form.
			jQuery( widgetID ).find( childClass ).submit();
			
			// Don't do anything else.
			return false;
		}

	// Output each item as a link instead of just a list item.
	}).autocomplete( 'instance' )._renderItem = function( ul, item ) {
		return jQuery( '<li>' )
		.append( "<a class='" + childClassHTML + "-link' href='" + item.value + "'>" + item.label + "</a>" )
		.appendTo( ul );
	};

	// When the form is submit, make sure it is pointing to a valid item.
	jQuery( widgetID ).find( childClass ).submit( function( event ) {
		
		// Remove the error text when the user attempts to submit, so we don't end up with lots of them.
		jQuery( widgetID ).find( childClass ).find( '.sjf_et-error' ).remove();

		// Grab the text in the search box.
		var needle = jQuery( widgetID + ' ' + childClass + '-input' ).val();

		// See if it's in the array of good values.
		var inArray = jQuery.inArray( needle, haystack );
		
		// Check for an empty for action.
		var formAction = jQuery( widgetID ).find( childClass ).attr( 'action' );

		// If something is amiss, either an empty form action or a bad spelling, warn the user.
		if( ( inArray == -1 ) || ( typeof formAction === 'undefined' ) ) {

			jQuery( this ).addClass( 'sjf_et-error' );
			jQuery( errorText ).hide().insertAfter( widgetID + ' ' + childClass ).fadeIn();
			return false;
		
		}

	});

	// Upon any change to the search box, remove the warning class.
	jQuery( widgetID ).find( childClass + '-input' ).on( 'input propertychange paste', function() {

		jQuery( widgetID ).find( childClass ).removeClass( 'sjf_et-error' );
		jQuery( widgetID ).find( childClass + '-error' ).fadeOut();

	});

}

/**
 * Popup
 */
function SJF_ET_PopupCB( widgetID ) {

	var childClass = SJF_ET.getChildClass( widgetID );

	// The actual popup dialogue thing.
	var popup = jQuery( childClass );

	// The button to close the popup.
	var close = jQuery( childClass + '-close' );

	// The name of the cookie associated with this popup.
	var cookie = jQuery( popup ).data( 'cookie' );

	// Move the popup to the end of the body so it stacks on top.
	jQuery( popup ).appendTo( 'body' ).hide().fadeIn();

	// When we click the close button or the overlay BG, close the popup and save a cookie.
	jQuery( [close, popup] ).each( function() {
		jQuery( this ).click( function( event ) {

			event.preventDefault();
			jQuery( popup ).fadeOut();
			document.cookie = cookie + '=1';
		});
	});

	// We want to be able to click the popup without triggering the call to fade it out.
	jQuery( childClass + '-inner' ).click( function( event ) {
		event.stopPropagation();
	});

}

/**
 * Slider
 *
 * The args were localized in the slider widget file.
 */
function SJF_ET_SliderCB( widgetID ) {

	if( typeof SJF_ET_Slider === 'undefined' ) { return false; }

	var childClass = SJF_ET.getChildClass( widgetID );
	
	// The configuration for the BX Slider.
	var args       = SJF_ET_Slider.args;
	var sliderArgs = jQuery.parseJSON( args );

	// Find the slider by class, instantiate it with the args from our php file.
	jQuery( widgetID ).find( childClass ).bxSlider( sliderArgs );

}

/**
 * Sortable
 */
function SJF_ET_SortableCB( widgetID ) {

	var childClass = SJF_ET.getChildClass( widgetID );

	if( ! jQuery().tablesorter ) { return false; }

	jQuery( widgetID ).find( childClass ).tablesorter();

}