<?php

/**
 * A wrapper for the WordPress Transients API.
 *
 * Provides public methods for getting and setting WP Transients.
 *
 * @package WordPress
 * @subpackage sjf-et
 * @since SJF ET 0.1
 */

/**
 * Abstracts the WP Transients API, with some addiitonal functionality.
 */
class SJF_Ecwid_Transients {

	/**
	 * Any time we do a transient, we check the request type and maybe flush transients.
	 * 
	 * @param string $request_type GET, POST, DELETE, PUT, etc.
	 * @param bool   $flush_rewrite_rules Do we want to also flush rewrite rules?
	 */
	public function __construct( $request_type = 'GET', $flush_rewrite_rules = TRUE ) {

		// Unless it's a GET, flush transients and maybe flush rewrites..
		if( $request_type != 'GET' ) {
			$this -> flush_transients();
			
			if( $flush_rewrite_rules ) {
				flush_rewrite_rules( TRUE );
			}
		
		}

	}

	/**
	 * Performs an SQL DELETE query to remove transients.  Also flushed rewrite rules.
	 */
	private function flush_transients() {

		// This is pretty rare, but we actually are writing SQL here!  Grab the wpdb class.
		global $wpdb;

		// Basically just a short version of the plugin namespace.
		$prefix = esc_sql( $this -> get_transient_prefix() );

		// The name of the options table for this install.
		$options = esc_sql( $wpdb -> options );

		$t  = esc_sql( "_transient_timeout_$prefix%" );

		$sql = $wpdb -> prepare (
			"
				SELECT option_name
				FROM $options
				WHERE option_name LIKE '%s'
			",
			$t
		);

		$transients = $wpdb -> get_col( $sql );

		// For each transient...
		foreach( $transients as $transient ) {

			// Strip away the WordPress prefix in order to arrive at the transient key.
			$key = str_replace( '_transient_timeout_', '', $transient );

			// Now that we have the key, use WordPress core to the delete the transient.
			delete_transient( $key );

		}

		// But guess what?  Sometimes transients are not in the DB, so we have to do this too:
		wp_cache_flush();

	}

	/**
	 * Grab the transient key for a request.
	 * 
	 * @param  string $request_url The url for an http request to ecwid.
	 * @return string The transient key for an http request to ecwid.
	 */
	public function get_transient_key( $request_url ) {

		// Grab the transient prefix to keep things namespaced to our plugin.
		$transient_prefix = $this -> get_transient_prefix();

		// Append the version number so that all of our transients are automatically dumped upon plugin update.
		$key = $request_url . '_' . SJF_ET_VERSION;

		// We md5 the key to keep it down to 32 chars.
		$key =  md5( $key );

		// Prepend the prefix so we can easily recognize it in the DB.
		$out = $transient_prefix . $key;

		return $out;

	}

	/**
	 * Get the transient prefix for our plugin.
	 * 
	 * @return string The transient prefix for our plugin.
	 */
	private function get_transient_prefix() {

		/**
		 * WordPress limits transient keys to as little as 40 chars in some situations,
		 * and we're going to add a 32-char md5 hash, so limit the prefix to 6 chars.
		 */
		$out = substr( SJF_Ecwid_Helpers::get_namespace() , 0, 7 );
		return $out;

	}

	/**
	 * A wrapper for the wordpress set_transient function.  Useful becuase it keeps us consistent on expiration length.
	 * 
	 * @param string $transient_key The key for this transient.
	 * @param string $content The blob of HTML that we are saving -- probably the result of a remote call.
	 */
	function set_transient( $transient_key, $content ) {

		// No matter what, this result will expire in one day.
		return set_transient( $transient_key, $content, DAY_IN_SECONDS );
	}

	/**
	 * A wrapper for the wordpress get_transient function. This is pretty pointless.
	 */
	public function get_transient( $transient_key ) {
		return get_transient( $transient_key );
	}

}